#!/usr/bin/env python3
####################################################################################
# satpy area visualizer
# Hilversum   , 2023/04/01       License GPL3          (c) Rob Alblas
####################################################################################
class carea:
  def __init__(self):
    self.head=None
    self.next=None
    self.name=None
    self.lon=None
    self.lat=None
    self.on=0
  def __iter__(self):
    node = self.head
    while node is not None:
        yield node
        node = node.next

def is_new_area(ln):
  sln=ln.split()
  if sln!=[] and sln[0][len(sln[0])-1:] == ':' and ln[0]!=' ':
    return sln[0][:len(sln[0])-1]
  else:
    return None

def add_item(items,item):
  if items!=None:
    if (items.head==None):
      items.head=item
    else:
      for p in items:
        if p.next==None:
          p.next=item
          break
  else:
    items=item
    items.head=item
  return items

def add_area(rareas,name,lat=None,lon=None):
  area=carea()
  area.name=name
  area.lat=lat
  area.lon=lon
  rareas=add_item(rareas,area)
  return rareas

def read_one_area(fp):
  flag=0
  for ln in iter(fp.readline,''):           # parse line by line
    sln=ln.split('\n')
    areaname=is_new_area(ln)
    if areaname!=None:
      area=carea()
      area.name=areaname
      pos=0
      for lnt in iter(fp.readline,''):           # parse line by line
        ln=lnt.split('\n')
        
        if is_new_area(lnt)!=None:
          fp.seek(pos)
          return area

        pos=fp.tell()
        slnx=ln[0].split('  ')
        if len(slnx)>1:
          if slnx[1]=='projection:':
            flag='p'
          elif slnx[1]!='':
            flag=0

          if flag=='p':
            sln=ln[0].split()
            if len(sln)>1:
              if sln[0]=='lon_0:':
                area.lon=float(sln[1])
              if sln[0]=='lat_0:':
                area.lat=float(sln[1])
              if sln[0]=='lat_ts:':
                area.lat=float(sln[1])
      return area
  return None

def read_all_areas():
  careas=None
  areafiles=get_area_file()
  for fs in areafiles:         # all area files (.yaml)
    print(fs)
    with open(fs) as fp:
      while True:
        area=read_one_area(fp)
        if area==None:
          break
        if careas!=None:
          for n in careas:
            if n.name==area.name:
              break
          if n.next==None:
            careas=add_item(careas,area)
        else:
          careas=add_item(careas,area)
      fp.close()
  careas=sorted(careas,key=lambda careas:careas.name)
  return careas

#             name        lat1,lon1  lat2,lon2
regio_list=[['Arctica'   ,( 65,-180),( 90,180)],
            ['N europe'  ,( 55, -20),( 70, 40)],
            ['europe'    ,( 32, -20),( 55, 40)],
            ['asia'      ,( -9,  56),( 51,158)],
            ['africa'    ,(-38, -25),( 32, 48)],
            ['N america' ,(  4,-169),( 70,-58)],
            ['S america' ,(-55, -87),( 15,-32)],
            ['Antarctica',(-90,-180),(-65,180)],
            ['rest'      ,(-90,-180),( 90,180)]]

class cregio:
  def __init__(self,name=None):
    self.head=None
    self.next=None
    self.name=name
    self.area=None # carea()
  def __iter__(self):
    node = self.head
    while node is not None:
        yield node
        node = node.next

# test if lat/lon is in regio 'regioname'
#     return regio if so, or 'None'
def in_regio(regio_list,regioname,lat,lon):
  if regioname!='rest':
    for regio in regio_list:
      if regio[0]==regioname:
        if lat>=regio[1][0] and lat <= regio[2][0] and \
          lon>=regio[1][1] and lon <= regio[2][1]:
          return True
        else:
          return False
  else:
    for regio in regio_list:
      if regio[0]!='rest':
        if lat>=regio[1][0] and lat <= regio[2][0] and \
           lon>=regio[1][1] and lon <= regio[2][1]:
          return False
      else:
        return regio
  return None


def create_regioarea(regio_list,areas):
  regios=None
  for regio in regio_list:                         # regio=[regiox,(lat1,lon1),(lat2,lon2)]
    r=cregio(regio[0])
    add_item(regios,r)

    if regios==None:
      regios=r
    for p in areas:
      if p.lat!=None and p.lon!=None:
        if in_regio(regio_list,regio[0],p.lat,p.lon): # test area in box of area[0]
          r.area=add_area(r.area,p.name,p.lat,p.lon)

  return regios

def pri_regios(ra):
  for r in ra:
    print(r.name)
    for a in r.area:
      print('  '+a.name)

#############################################################
# gui part
#
import tkinter as tk
from tkinter import *
from PIL import ImageTk, Image
cvsx=None
frame=None
#fact=2.5
def track_viewer(fn):
  global img # anders img opgeruimd na func!
  global cvsx
  global frame
  if fn==None:
    if frame:
      frame.destroy()
      frame=None
      cvsx=None
    return

  width=int(360*fact)
  height=int(180*fact)

  # create frame with canvas, or use existing one
  if cvsx==None:
    frame=Frame()
    cvsx = tk.Canvas(frame, bg="grey90", height=height, width=width)
    cvsx.grid()
    frame.pack()

  # clear canvas, then add background image again 
  if cvsx != None:
    cvsx.delete('all')
  try:
    img1=Image.open(fn)
  except:
    if frame:
      frame.destroy()
    frame=None
    cvsx=None

    print(fn+" doesn't exist.")
    return None,None,None

  img1=img1.resize((width,height))
  img = ImageTk.PhotoImage(img1)
  cvsx.create_image(0,0,image=img,anchor="nw")

  # Add lon/lat grid
  step=30 # degrees, lat and lon
  for lat in range(-90+step,90,step):
    cvsx.create_line(0,(lat+90)*fact,360*fact,(lat+90)*fact,width=1)
  for lon in range(0+step,360,step):
    cvsx.create_line(lon*fact,0,lon*fact,180*fact,width=1)

  return cvsx,width,height

from satpy.resample import get_area_def
from satpy.resample import get_area_file
import pyresample
#entry1=None
def draw_area(cvsx,sel_area,width,height):
  global entry1
  entry1.delete(0,'end')
  try:
    sarea=get_area_def(sel_area)
  except:
    entry1.insert(0,'No area "'+str(sel_area)+'" found.')
    return

  try:
    arr=pyresample.geometry.BaseDefinition.get_bbox_lonlats(sarea)
  except:
    entry1.insert(0,"Can't create bbox for "+str(sel_area))
    return

  lons=arr[0]
  lats=arr[1]
  m=0
  for lon in lons:
    lat=lats[m]
    n=0
    for lon1 in lon:
      if abs(lon1) > 1000:
        continue
      lat1=lat[n]
      if abs(lat1) > 1000:
        continue
      ilon=int((lon1+180)*fact)
      ilat=height-1-int((lat1+90)*fact)

      if n>0:
        if abs(lon1-plon1)<300:
          cvsx.create_line(ilon0, ilat0, ilon, ilat,width=1,fill=areaclr)
      plon1=lon1
      ilon0=ilon
      ilat0=ilat
      ilon0=ilon
      ilat0=ilat
      n+=1
    m+=1

def plot_area(sel_area=None):
  (cvsx,width,height)=track_viewer(earth_image)
  if cvsx==None:
    return

  if sel_area!=None:
    draw_area(cvsx,sel_area,width,height)
    return

# geeft vorige waarde...
def select_area():
  plot_area(sel_area.get())

def sel_xx():
  print(sel_area.get())
#  for p in regio_area:
#    for q in p.area:
#      print(str(q.name)+'  '+str(q.name))

def create_gui(regio_area):
  global sel_area,entry1
  wnd = tk.Tk()
  wnd.title('Satpy area visualizer')


  # add canvas
  (cvsx,width,height)=track_viewer(earth_image)

  if use_buttons:
    x=1
    y=1
    sel_area = tk.StringVar(value="")
    frame0=Frame(wnd)
    for p in regio_area:
      if p.area != None:
        frame1=Frame(frame0,highlightbackground="blue",highlightthickness=2)
        lab=tk.Label(frame1,text=p.name,anchor='nw').grid(column=x,row=0,sticky='W')
        for q in p.area:
  #        but=Checkbutton(frame1,text=q.name,command=sel_xx,variable=q.name,onvalue=1,anchor='nw').grid(column=x,row=y,sticky='W')
          but=Radiobutton(frame1,value=q.name,text=q.name,variable=sel_area,command=select_area,anchor='nw').grid(column=x,row=y,sticky='W')
          y+=1
          if y>15:
            x+=1
            y=1
        x+=1
        y=1
        frame1.pack(side=tk.LEFT,anchor='nw')
  else:
    frame0=Frame(wnd)
    alabel=tk.Label(frame0,text='Area')
    sel_area = tk.StringVar(value="---")

    # make area menu
    amenubutton = tk.Menubutton(frame0, textvariable=sel_area, indicatoron=True,borderwidth=1, relief="raised")
    amenu = tk.Menu(amenubutton, tearoff=True)
    amenubutton.configure(menu=amenu)

    for p in regio_area:
      a_submenu = tk.Menu(amenu, tearoff=True)
      amenu.add_cascade(label=p.name, menu=a_submenu)
      for q in p.area:
        a_submenu.add_radiobutton(value=q.name, label=q.name, variable=sel_area,command=select_area)

    amenu=amenubutton

    # place area button
    alabel.grid(column=1,row=1)
    amenu.grid(column=1,row=2)
    frame0.pack(side=tk.TOP,anchor='w')

  frame0.pack(side=tk.TOP)

  frame2=Frame(wnd)
  entry1=Entry(frame2,font=('Arial', 12, 'bold'),width=40)
  entry1.grid()
  frame2.pack(side=tk.LEFT,anchor='sw')

  return wnd

#earth_image='earthmap.gif'
import argparse
parser = argparse.ArgumentParser()
parser.add_argument('-s'        ,default=2.5            , help="size (multiplier fact.) image (1=org. size)")
parser.add_argument('-f'        ,default='earthmap.gif' , help="plate carree image file")
parser.add_argument('-ac'       ,default='blue'         , help="colour area")
parser.add_argument('-m'        ,default=False,nargs='?',const=True , help="use menu instead of buttons")
args = parser.parse_args()

fact=float(args.s)
earth_image=args.f
areaclr=args.ac
use_buttons=not args.m

areas=read_all_areas()
regio_area=create_regioarea(regio_list,areas)

wnd=create_gui(regio_area)
wnd.mainloop()
